%define anolis_release 1
%global toolchain clang
%undefine _include_frame_pointers
%global maj_ver 16
%global libcxx_version %{maj_ver}.0.6
%global libcxx_srcdir libcxx-%{libcxx_version}.src
%global libcxxabi_srcdir libcxxabi-%{libcxx_version}.src
%global libunwind_srcdir libunwind-%{libcxx_version}.src

Name:		libcxx
Version:	%{libcxx_version}
Release:	%{anolis_release}%{?dist}
Summary:	C++ standard library targeting C++11
License:	Apache-2.0 WITH LLVM-exception OR MIT OR NCSA
URL:		http://libcxx.llvm.org/
Source0:	https://github.com/llvm/llvm-project/releases/download/llvmorg-%{libcxx_version}/%{libcxx_srcdir}.tar.xz
Source1:	https://github.com/llvm/llvm-project/releases/download/llvmorg-%{libcxx_version}/%{libcxxabi_srcdir}.tar.xz
Source2:	https://github.com/llvm/llvm-project/releases/download/llvmorg-%{libcxx_version}/%{libunwind_srcdir}.tar.xz
Source5:	CMakeLists.txt
Patch0:		standalone.patch

BuildRequires:	clang
BuildRequires:	llvm-devel cmake ninja-build
# We need python3-devel for %%py3_shebang_fix
BuildRequires:  python3-devel
# For documentation
BuildRequires:	python3-sphinx

Requires: libcxxabi = %{version}-%{release}

%description
libc++ is a new implementation of the C++ standard library, targeting C++11.

%package doc
Summary:        Documentation files for %{name}
Requires:       %{name} = %{version}-%{release}
BuildArch:      noarch

%description    doc
The %{name}-doc package contains documentation files for %{name}.

%package devel
Summary:	Headers and libraries for libcxx devel
Requires:	%{name} = %{version}-%{release}
Requires:	libcxxabi-devel = %{version}

%description devel
%{summary}.

%package static
Summary:	Static libraries for libcxx

%description static
%{summary}.

%package -n libcxxabi
Summary:	Low level support for a standard C++ library

%description -n libcxxabi
libcxxabi provides low level support for a standard C++ library.

%package -n libcxxabi-doc
Summary:        Documentation files for %{name}
Requires:       libcxxabi = %{version}-%{release}
BuildArch:      noarch

%description -n libcxxabi-doc
The %{name}-doc package contains documentation files for %{name}.

%package -n libcxxabi-devel
Summary:	Headers and libraries for libcxxabi devel
Requires:	libcxxabi = %{version}-%{release}

%description -n libcxxabi-devel
%{summary}.

%package -n libcxxabi-static
Summary:	Static libraries for libcxxabi

%description -n libcxxabi-static
%{summary}.

%package -n llvm-libunwind
Summary:    LLVM libunwind

%description -n llvm-libunwind

LLVM libunwind is an implementation of the interface defined by the HP libunwind
project. It was contributed Apple as a way to enable clang++ to port to
platforms that do not have a system unwinder. It is intended to be a small and
fast implementation of the ABI, leaving off some features of HP's libunwind
that never materialized (e.g. remote unwinding).

%package -n llvm-libunwind-devel
Summary:    LLVM libunwind development files
Provides:   libunwind(major) = %{maj_ver}
Requires:   llvm-libunwind = %{version}-%{release}

%description -n llvm-libunwind-devel
Unversioned shared library for LLVM libunwind

%package -n llvm-libunwind-static
Summary: Static library for LLVM libunwind

%description -n llvm-libunwind-static
%{summary}.

%package -n llvm-libunwind-doc
Summary:    libunwind documentation
# jquery.js and langage_data.js are used in the HTML doc and under BSD License
License:    BSD AND (Apache-2.0 WITH LLVM-exception OR NCSA OR MIT)
Buildarch:  noarch

%description -n llvm-libunwind-doc
Documentation for LLVM libunwind

%prep
%setup -T -q -b 0 -n %{libcxx_srcdir}
%setup -T -q -b 1 -n %{libcxxabi_srcdir}
%setup -T -q -b 2 -n %{libunwind_srcdir}
%setup -T -c -n build
 
cp %{SOURCE5} .
mv ../%{libcxx_srcdir} libcxx
mv ../%{libcxxabi_srcdir} libcxxabi
mv ../%{libunwind_srcdir} libunwind
%autopatch -p1
 
%py3_shebang_fix libcxx/utils/

%build

%cmake -GNinja \
	-DCMAKE_BUILD_TYPE=RelWithDebInfo \
	-DCMAKE_MODULE_PATH=%{_libdir}/cmake/llvm \
	-DLIBCXX_LIBDIR_SUFFIX:STRING=64 \
	-DLIBCXXABI_LIBDIR_SUFFIX:STRING=64 \
	-DLIBUNWIND_LIBDIR_SUFFIX:STRING=64 \
	-DLIBCXX_INCLUDE_BENCHMARKS=OFF \
	-DLIBCXX_STATICALLY_LINK_ABI_IN_STATIC_LIBRARY=ON \
	-DLIBCXX_ENABLE_ABI_LINKER_SCRIPT=ON \
	-DLLVM_BUILD_DOCS=ON \
	-DLLVM_ENABLE_SPHINX=ON \
	-DLIBUNWIND_INCLUDE_DOCS=ON \
	-DLIBUNWIND_INSTALL_INCLUDE_DIR=%{_includedir}/llvm-libunwind \
	-DLIBUNWIND_INSTALL_SPHINX_HTML_DIR=%{_pkgdocdir}/html

%cmake_build

%install

%cmake_install

# We can't install the unversionned path on default location because that would conflict with
# libunwind
#
# The versionned path has a different soname (libunwind.so.1 compared to
# libunwind.so.8) so they can live together in %%{_libdir}
#
# ABI wise, even though llvm-libunwind's library is named libunwind, it doesn't
# have the exact same ABI as gcc's libunwind (it actually provides a subset).
#
# For the same reason, please do not generate abi dump for %%{_libdir}/*.so.*, as they will
# conflict with libunwind
rm %{buildroot}%{_libdir}/libunwind.so
mkdir -p %{buildroot}/%{_libdir}/llvm-unwind/

pushd %{buildroot}/%{_libdir}/llvm-unwind
ln -s ../libunwind.so.1.0 libunwind.so
popd

%files
%license libcxx/LICENSE.TXT
%{_libdir}/libc++.so.*

%files doc
%doc libcxx/CREDITS.TXT libcxx/TODO.TXT

%files devel
%{_includedir}/c++/
%exclude %{_includedir}/c++/v1/cxxabi.h
%exclude %{_includedir}/c++/v1/__cxxabi_config.h
%{_libdir}/libc++.so

%files static
%license libcxx/LICENSE.TXT
%{_libdir}/libc++.a
%{_libdir}/libc++experimental.a

%files -n libcxxabi
%license libcxxabi/LICENSE.TXT
%{_libdir}/libc++abi.so.*

%files -n libcxxabi-doc
%doc libcxxabi/CREDITS.TXT

%files -n libcxxabi-devel
%{_includedir}/c++/v1/cxxabi.h
%{_includedir}/c++/v1/__cxxabi_config.h
%{_libdir}/libc++abi.so

%files -n libcxxabi-static
%{_libdir}/libc++abi.a

%files -n llvm-libunwind
%license libunwind/LICENSE.TXT
%{_libdir}/libunwind.so.1
%{_libdir}/libunwind.so.1.*

%files -n llvm-libunwind-devel
%{_includedir}/llvm-libunwind/__libunwind_config.h
%{_includedir}/llvm-libunwind/libunwind.h
%{_includedir}/llvm-libunwind/libunwind.modulemap
%{_includedir}/llvm-libunwind/mach-o/compact_unwind_encoding.h
%{_includedir}/llvm-libunwind/mach-o/compact_unwind_encoding.modulemap
%{_includedir}/llvm-libunwind/unwind.h
%{_includedir}/llvm-libunwind/unwind_arm_ehabi.h
%{_includedir}/llvm-libunwind/unwind_itanium.h
%dir %{_libdir}/llvm-unwind
%{_libdir}/llvm-unwind/libunwind.so

%files -n llvm-libunwind-static
%{_libdir}/libunwind.a

%files -n llvm-libunwind-doc
%license libunwind/LICENSE.TXT
%doc %{_pkgdocdir}/html

%changelog
* Tue Aug 8 2023 Funda Wang <fundawang@yeah.net> - 16.0.6-1
- New version 16.0.6
- Merge llvm-libunwind into libcxx

* Thu May 18 2023 Funda Wang <fundawang@yeah.net> - 16.0.4-1
- New version 16.0.4

* Fri Apr 14 2023 Funda Wang <fundawang@yeah.net> - 16.0.1-1
- New version 16.0.1

* Wed Feb 1 2023 chen_yu_ao <m15927509890@163.com> - 15.0.7-1
- Init package from upstream
